package com.brilliance.swift.spi;

import com.brilliance.swift.util.ObjectUtil;

import java.lang.ref.Reference;
import java.lang.ref.SoftReference;
import java.lang.ref.WeakReference;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 对ConcurrentHashMap的封装，目的是通过值的引用封装，从而在JVM内存吃紧时GC时会自动释放内存。对于用户操作键值对却是无感的。
 * <p><b>注意：</b>引用的使用意味着不能保证放置到Map中的键值对随后可用。垃圾收集器可能会随时丢弃引用，因此可能会出现未知线程正在悄悄删除键值对。</p>
 *  <p>支持ReferenceType.SOFT或ReferenceType.WEAK类型引用，默认为SOFT。</p>
 */
public class ConcurrentReferenceMap<T, S> {
    private static final ReferenceType DEFAULT_REFERENCE_TYPE = ReferenceType.SOFT;
    private static final int DEFAULT_INITIAL_CAPACITY = 16;
    private final ConcurrentHashMap<T, Reference<S>> cache;
    private final ReferenceType referenceType;

    /**
     * 空构造ConcurrentReferenceMap。使用默认引用类型SOFT和默认缓存初始容量16
     */
    public ConcurrentReferenceMap() {
        this(DEFAULT_REFERENCE_TYPE, DEFAULT_INITIAL_CAPACITY);
    }

    /**
     * 构造一个初始容量为initialCapacity的ConcurrentReferenceMap。使用默认引用类型SOFT
     * @param initialCapacity 初始容量大小
     */
    public ConcurrentReferenceMap(int initialCapacity) {
        this(DEFAULT_REFERENCE_TYPE, initialCapacity);
    }

    /**
     * 构造一个指定引用类型和初始容量的ConcurrentReferenceMap。
     * @param referenceType 引用类型（ReferenceType.SOFT或ReferenceType.WEAK）
     * @param initialCapacity
     */
    public ConcurrentReferenceMap(ReferenceType referenceType, int initialCapacity) {
        this.cache = new ConcurrentHashMap<>(initialCapacity);
        this.referenceType = referenceType;
    }

    public S get(Object key) {
        Reference<S> sReference = cache.get(key);
        if (ObjectUtil.isNull(sReference)) {
            return null;
        }
        return sReference.get();
    }

    public S getOrDefault(Object key, S defaultValue) {
        S v;
        return (v = get(key)) == null ? defaultValue : v;
    }

    public S put(T key, S value) {
        Reference<S> oldref = null;
        if (value instanceof Reference) {
            oldref = cache.put(key, (Reference<S>) value);
        } else {
            oldref = cache.put(key, referenceType.generateReference(value));
        }
        return ObjectUtil.isNull(oldref) ? null : oldref.get();
    }

    public void clear(){
        cache.clear();
    }

    /**
     * 强制清理值为空的键值对
     */
    public void clearUnreferencedEntries() {
        cache.entrySet().removeIf(entry -> ObjectUtil.isNull(entry.getValue()));
    }

    public enum ReferenceType {
        SOFT {
            @Override
            Reference generateReference(Object value) {
                return new SoftReference(value);
            }
        }, WEAK {
            @Override
            Reference generateReference(Object value) {
                return new WeakReference(value);
            }
        };

        abstract Reference generateReference(Object value);
    }
}
