/*
 * Copyright 2002-2021 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.brilliance.swift.util;

import com.brilliance.swift.exception.InstantiateException;
import com.brilliance.swift.spi.ConcurrentReferenceMap;

import java.lang.reflect.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 反射工具
 */
public class ReflectionUtil {

    /**
     * 预构建的 MethodFilter 匹配所有未在java.lang.Object声明的非桥非合成方法
     */
    public static final MethodFilter USER_DECLARED_METHODS =
            (method -> !method.isBridge() && !method.isSynthetic());

    /**
     * 匹配所有非静态、非最终字段的预构建 FieldFilter.
     */
    public static final FieldFilter COPYABLE_FIELDS =
            (field -> !(Modifier.isStatic(field.getModifiers()) || Modifier.isFinal(field.getModifiers())));

    private static final Method[] EMPTY_METHOD_ARRAY = new Method[0];

    private static final Field[] EMPTY_FIELD_ARRAY = new Field[0];

    private static final ConcurrentReferenceMap<Class<?>, Method[]> declaredMethodsCache = new ConcurrentReferenceMap<Class<?>, Method[]>(256);

    private static final ConcurrentReferenceMap<Class<?>, Field[]> declaredFieldsCache = new ConcurrentReferenceMap<Class<?>, Field[]>(256);

    /**
     * 获取给定类和参数的可访问构造函数
     *
     * @param clazz          要检查的 clazz
     * @param parameterTypes 所需构造函数的参数类型
     * @return 构造函数引用
     * @throws NoSuchMethodException 如果不存在这样的构造函数
     */
    public static <T> Constructor<T> accessibleConstructor(Class<T> clazz, Class<?>... parameterTypes)
            throws NoSuchMethodException {

        Constructor<T> ctor = clazz.getDeclaredConstructor(parameterTypes);
        makeAccessible(ctor);
        return ctor;
    }

    /**
     * 使给定的方法可访问，必要时显式设置它可访问。{@code setAccessible(true)}方法仅在实际需要时调用
     *
     * @param method 使方法可访问
     */
    public static void makeAccessible(Method method) {
        if ((!Modifier.isPublic(method.getModifiers()) ||
                !Modifier.isPublic(method.getDeclaringClass().getModifiers())) && !method.isAccessible()) {
            method.setAccessible(true);
        }
    }

    /**
     * 使给定的构造函数可访问，必要时显式设置它可访问。
     * setAccessible(true)方法仅在实际需要时调用，以避免与 JVM SecurityManager（如果活动）发生不必要的冲突
     *
     * @param ctor 使构造函数可访问
     */
    public static void makeAccessible(Constructor<?> ctor) {
        if ((!Modifier.isPublic(ctor.getModifiers()) ||
                !Modifier.isPublic(ctor.getDeclaringClass().getModifiers())) && !ctor.isAccessible()) {
            ctor.setAccessible(true);
        }
    }

    /**
     * 使给定的字段可访问，必要时明确设置它可访问。
     * setAccessible(true)方法仅在实际需要时调用，以避免与 JVM SecurityManager（如果活动）发生不必要的冲突
     *
     * @param field 使字段可访问
     */
    public static void makeAccessible(Field field) {
        if ((!Modifier.isPublic(field.getModifiers()) ||
                !Modifier.isPublic(field.getDeclaringClass().getModifiers()) ||
                Modifier.isFinal(field.getModifiers())) && !field.isAccessible()) {
            field.setAccessible(true);
        }
    }

    /**
     * 在给定类中的所有本地声明的字段上调用给定的回调
     *
     * @param clazz 要分析的目标类
     * @param fc    为每个字段调用的回调
     */
    public static void doWithLocalFields(Class<?> clazz, FieldCallback fc) {
        for (Field field : getDeclaredFields(clazz)) {
            try {
                fc.doWith(field);
            } catch (IllegalAccessException ex) {
                throw new IllegalStateException("Not allowed to access field '" + field.getName() + "': " + ex);
            }
        }
    }

    /**
     * 在目标类中的所有字段上调用给定的回调，沿着类层次结构向上获取所有声明的字段。
     *
     * @param clazz 要分析的目标类
     * @param fc    为每个字段调用的回调
     */
    public static void doWithFields(Class<?> clazz, FieldCallback fc) {
        doWithFields(clazz, fc, null);
    }

    /**
     * 在目标类中的所有字段上调用给定的回调，沿着类层次结构向上获取所有声明的字段。
     *
     * @param clazz 要分析的目标类
     * @param fc    为每个字段调用的回调
     * @param ff    确定要应用回调的字段的过滤器
     */
    public static void doWithFields(Class<?> clazz, FieldCallback fc, FieldFilter ff) {
        Class<?> targetClass = clazz;
        do {
            Field[] fields = getDeclaredFields(targetClass);
            for (Field field : fields) {
                if (ff != null && !ff.matches(field)) {
                    continue;
                }
                try {
                    fc.doWith(field);
                } catch (IllegalAccessException ex) {
                    throw new IllegalStateException("Not allowed to access field '" + field.getName() + "': " + ex);
                }
            }
            targetClass = targetClass.getSuperclass();
        }
        while (targetClass != null && targetClass != Object.class);
    }

    /**
     * 浅拷贝。给定源对象和目标，它们必须是同一个类或子类，复制所有字段，包括继承的字段。 旨在处理具有公共无参数构造函数的对象
     *
     * @param src  源对象
     * @param dest 目标对象
     */
    public static void shallowCopyFieldState(final Object src, final Object dest) {
        if (!src.getClass().isAssignableFrom(dest.getClass())) {
            throw new IllegalArgumentException("Destination class [" + dest.getClass().getName() +
                    "] must be same or subclass as source class [" + src.getClass().getName() + "]");
        }
        doWithFields(src.getClass(), field -> {
            makeAccessible(field);
            Object srcValue = field.get(src);
            field.set(dest, srcValue);
        }, COPYABLE_FIELDS);
    }

    /**
     * 对给定类的所有匹配方法执行给定的回调操作，如本地声明或等效的（例如给定类实现的基于 Java 8 的接口上的默认方法）。
     *
     * @param clazz 内省的类
     * @param mc    为每个方法调用的回调
     */
    public static void doWithLocalMethods(Class<?> clazz, MethodCallback mc) {
        Method[] methods = getDeclaredMethods(clazz, false);
        for (Method method : methods) {
            try {
                mc.doWith(method);
            } catch (IllegalAccessException ex) {
                throw new IllegalStateException("Not allowed to access method '" + method.getName() + "': " + ex);
            }
        }
    }

    /**
     * 对给定类和超类的所有匹配方法执行给定的回调操作。</br>
     * <strong>出现在子类和超类上的相同命名方法将出现两次，除非被ReflectionUtils.MethodFilter排除。</strong>
     *
     * @param clazz 内省的类
     * @param mc    为每个方法调用的回调
     */
    public static void doWithMethods(Class<?> clazz, MethodCallback mc) {
        doWithMethods(clazz, mc, null);
    }

    public static void doWithMethods(Class<?> clazz, MethodCallback mc, MethodFilter mf) {
        // 继续缓存继承层次结构.
        Method[] methods = getDeclaredMethods(clazz, false);
        for (Method method : methods) {
            if (mf != null && !mf.matches(method)) {
                continue;
            }
            try {
                mc.doWith(method);
            } catch (IllegalAccessException ex) {
                throw new IllegalStateException("Not allowed to access method '" + method.getName() + "': " + ex);
            }
        }
        if (clazz.getSuperclass() != null && (mf != USER_DECLARED_METHODS || clazz.getSuperclass() != Object.class)) {
            doWithMethods(clazz.getSuperclass(), mc, mf);
        } else if (clazz.isInterface()) {
            for (Class<?> superIfc : clazz.getInterfaces()) {
                doWithMethods(superIfc, mc, mf);
            }
        }
    }

    /**
     * 获取子类和所有父类上的所有声明方法。首先包含子类方法。
     *
     * @param leafClass 要内省的类
     */
    public static Method[] getAllDeclaredMethods(Class<?> leafClass) {
        final List<Method> methods = new ArrayList<>(20);
        doWithMethods(leafClass, methods::add);
        return methods.toArray(EMPTY_METHOD_ARRAY);
    }

    /**
     * 获取子类和所有父类的唯一声明方法集。 叶类方法首先被包含在内，在遍历父类层次结构时，任何找到的签名与已包含的方法相匹配的方法都将被过滤掉
     *
     * @param leafClass 要内省的类
     * @return 子类和所有父类的唯一声明方法集
     */
    public static Method[] getUniqueDeclaredMethods(Class<?> leafClass) {
        return getUniqueDeclaredMethods(leafClass, null);
    }

    /**
     * 获取子类和所有父类的唯一声明方法集。 叶类方法首先被包含在内，在遍历父类层次结构时，任何找到的签名与已包含的方法相匹配的方法都将被过滤掉
     *
     * @param leafClass 要内省的类
     * @param mf        确定要考虑的方法的过滤器
     * @return 子类和所有父类的唯一声明方法集
     */
    public static Method[] getUniqueDeclaredMethods(Class<?> leafClass, MethodFilter mf) {
        final List<Method> methods = new ArrayList<>(20);
        doWithMethods(leafClass, method -> {
            boolean knownSignature = false;
            Method methodBeingOverriddenWithCovariantReturnType = null;
            for (Method existingMethod : methods) {
                if (method.getName().equals(existingMethod.getName()) &&
                        method.getParameterCount() == existingMethod.getParameterCount() &&
                        Arrays.equals(method.getParameterTypes(), existingMethod.getParameterTypes())) {
                    // 如果新发现的方法返回类型是已存在方法的子类型，则覆盖
                    if (existingMethod.getReturnType() != method.getReturnType() &&
                            existingMethod.getReturnType().isAssignableFrom(method.getReturnType())) {
                        methodBeingOverriddenWithCovariantReturnType = existingMethod;
                    } else {
                        knownSignature = true;
                    }
                    break;
                }
            }
            if (methodBeingOverriddenWithCovariantReturnType != null) {
                methods.remove(methodBeingOverriddenWithCovariantReturnType);
            }
            if (!knownSignature) {
                methods.add(method);
            }
        }, mf);
        return methods.toArray(EMPTY_METHOD_ARRAY);
    }

    /**
     * Class.getDeclaredMethods()的变体，它使用本地缓存以避免 JVM 的 SecurityManager 检查和新的 Method 实例。
     * 此外，它还包括来自本地实现的接口的 Java 8 默认方法，因为这些方法实际上被视为已声明的方法。
     *
     * @param clazz 内省的类
     * @return 缓存的方法数组
     */
    public static Method[] getDeclaredMethods(Class<?> clazz) {
        return getDeclaredMethods(clazz, true);
    }

    /**
     * 针对不带参数的提供的目标对象调用指定的Method 。 调用静态Method时，目标对象可以为null
     *
     * @param method 要调用的方法
     * @param target 调用方法的目标对象
     * @param args   调用方法的参数
     * @return 调用结果，如果有的话
     */
    public static Object invokeMethod(Method method, Object target, Object... args) {
        try {
            return method.invoke(target, args);
        } catch (Exception ex) {
            handleReflectionException(ex);
        }
        throw new IllegalStateException("代码不应该执行到这!!!");
    }

    public static void handleReflectionException(Exception ex) {
        if (ex instanceof NoSuchMethodException) {
            throw new IllegalStateException("Method not found: " + ex.getMessage());
        }
        if (ex instanceof IllegalAccessException) {
            throw new IllegalStateException("Could not access method or field: " + ex.getMessage());
        }
        if (ex instanceof InvocationTargetException) {
            handleInvocationTargetException((InvocationTargetException) ex);
        }
        if (ex instanceof RuntimeException) {
            throw (RuntimeException) ex;
        }
        throw new UndeclaredThrowableException(ex);
    }

    public static void handleInvocationTargetException(InvocationTargetException exs) {
        Throwable ex = exs.getTargetException();
        if (ex instanceof RuntimeException) {
            throw (RuntimeException) ex;
        }
        if (ex instanceof Error) {
            throw (Error) ex;
        }
        throw new UndeclaredThrowableException(ex);
    }

    /**
     * 试图找到一个Method上与提供的名称和任何参数提供的类。 搜索直到Object所有超类。
     * 如果找不到Method则返回null
     *
     * @param clazz 内省的类
     * @param name  方法的名称
     * @return Method 对象，如果没有找到则为null
     */
    public static Method findMethod(Class<?> clazz, String name) {
        return findMethod(clazz, name, new Class<?>[0]);
    }

    /**
     * 尝试使用提供的名称和参数类型在提供的类上查找Method 。 搜索直到Object所有超类。
     * 如果找不到Method则返回null
     *
     * @param clazz      内省的类
     * @param name       方法的名称
     * @param paramTypes 方法的参数类型（可以为null以指示任何签名）
     * @return Method 对象，如果没有找到则为null
     */
    public static Method findMethod(Class<?> clazz, String name, Class<?>... paramTypes) {
        Class<?> searchType = clazz;
        while (searchType != null) {
            Method[] methods = (searchType.isInterface() ? searchType.getMethods() : getDeclaredMethods(searchType));
            for (Method method : methods) {
                if (name.equals(method.getName()) &&
                        (paramTypes == null || Arrays.equals(paramTypes, method.getParameterTypes()))) {
                    return method;
                }
            }
            searchType = searchType.getSuperclass();
        }
        return null;
    }

    private static Field[] getDeclaredFields(Class<?> clazz) {
        Field[] result = declaredFieldsCache.get(clazz);
        if (result == null) {
            try {
                result = clazz.getDeclaredFields();
                declaredFieldsCache.put(clazz, (result.length == 0 ? EMPTY_FIELD_ARRAY : result));
            } catch (Throwable ex) {
                throw new IllegalStateException("Failed to introspect Class [" + clazz.getName() +
                        "] from ClassLoader [" + clazz.getClassLoader() + "]", ex);
            }
        }
        return result;
    }

    private static Method[] getDeclaredMethods(Class<?> clazz, boolean defensive) {
        Method[] result = declaredMethodsCache.get(clazz);
        if (result == null) {
            try {
                Method[] declaredMethods = clazz.getDeclaredMethods();
                List<Method> defaultMethods = findConcreteMethodsOnInterfaces(clazz);
                if (defaultMethods != null) {
                    result = new Method[declaredMethods.length + defaultMethods.size()];
                    System.arraycopy(declaredMethods, 0, result, 0, declaredMethods.length);
                    int index = declaredMethods.length;
                    for (Method defaultMethod : defaultMethods) {
                        result[index] = defaultMethod;
                        index++;
                    }
                } else {
                    result = declaredMethods;
                }
                declaredMethodsCache.put(clazz, (result.length == 0 ? EMPTY_METHOD_ARRAY : result));
            } catch (Throwable ex) {
                throw new IllegalStateException("Failed to introspect Class [" + clazz.getName() +
                        "] from ClassLoader [" + clazz.getClassLoader() + "]", ex);
            }
        }
        return (result.length == 0 || !defensive) ? result : result.clone();
    }

    private static List<Method> findConcreteMethodsOnInterfaces(Class<?> clazz) {
        List<Method> result = null;
        for (Class<?> ifc : clazz.getInterfaces()) {
            for (Method ifcMethod : ifc.getMethods()) {
                if (!Modifier.isAbstract(ifcMethod.getModifiers())) {
                    if (result == null) {
                        result = new ArrayList<>();
                    }
                    result.add(ifcMethod);
                }
            }
        }
        return result;
    }

    public static <T> T instantiateClass(Constructor<T> ctor, Object... args) {
        try {
            ReflectionUtil.makeAccessible(ctor);
            return ctor.newInstance(args);
        } catch (InstantiationException ex) {
            throw new InstantiateException(ctor, "Is it an abstract class?", ex);
        } catch (IllegalAccessException ex) {
            throw new InstantiateException(ctor, "Is the constructor accessible?", ex);
        } catch (IllegalArgumentException ex) {
            throw new InstantiateException(ctor, "Illegal arguments for constructor", ex);
        } catch (InvocationTargetException ex) {
            throw new InstantiateException(ctor, "Constructor threw exception", ex.getTargetException());
        }
    }

    @FunctionalInterface
    public interface FieldCallback {

        /**
         * 使用给定字段执行操作.
         *
         * @param field 待操作的字段
         */
        void doWith(Field field) throws IllegalArgumentException, IllegalAccessException;
    }

    @FunctionalInterface
    public interface MethodCallback {
        /**
         * Perform an operation using the given method.
         *
         * @param method the method to operate on
         */
        void doWith(Method method) throws IllegalArgumentException, IllegalAccessException;
    }

    @FunctionalInterface
    public interface MethodFilter {

        /**
         * 确定给定的方法是否匹配.
         *
         * @param method 检查的方法
         */
        boolean matches(Method method);

        /**
         * 基于此过滤器和提供的过滤器创建一个复合过滤器。如果此过滤器不匹配，则不会应用下一个过滤器。
         * <p>如果此过滤器不匹配，则不会应用下一个过滤器。
         *
         * @param next 下一个MethodFilter
         * @return 复合MethodFilter
         * @throws IllegalArgumentException 如果 MethodFilter 参数为null
         */
        default MethodFilter and(MethodFilter next) {
            return method -> matches(method) && next.matches(method);
        }
    }

    @FunctionalInterface
    public interface FieldFilter {

        /**
         * 确定给定的字段是否匹配。
         *
         * @param field 待检测字段
         */
        boolean matches(Field field);

        /**
         * 基于此过滤器和提供的过滤器创建一个复合过滤器。如果此过滤器不匹配，则不会应用下一个过滤器。
         *
         * @param next 下一个FieldFilter
         * @return 复合FieldFilter
         * @throws IllegalArgumentException 如果 FieldFilter 参数为null
         */
        default FieldFilter and(FieldFilter next) {
            return field -> matches(field) && next.matches(field);
        }
    }
}
